/*
 * GLCD.c
 *
 * TG12864 GLCD driver for EK-TM4C1294XL
 *
 *  Created on: 2017/04/10
 *  Updated on: 2018/12/25
 *      Author: Aki Yamane (Adwin Corporation)
 */

#include <string.h>
#include <stdbool.h>
#include <unistd.h>
#include <inc/hw_memmap.h>
#include <driverlib/gpio.h>
#include <driverlib/sysctl.h>
#include "GLCD.h"

#include "font5x7.h"

#define GLCD_CS1    0b01
#define GLCD_CS2    0b10
#define GLCD_CMD    0
#define GLCD_DAT    1

#define GLCD_RST_H GPIOPinWrite(GPIO_PORTG_BASE,GPIO_PIN_1,0xff)
#define GLCD_RST_L GPIOPinWrite(GPIO_PORTG_BASE,GPIO_PIN_1,0)
#define GLCD_CS(X) GPIOPinWrite(GPIO_PORTH_BASE,0b11,(X))

#define GLCD_ADDR 0b01000000
#define GLCD_PAGE 0b10111000
#define GLCD_LINE 0b11000000

// 1 loop = 0.133us
#define LOOP_WAIT_30MS  300000
#define LOOP_WRITE_TIMING 30 // min:15

void GLCD_write(char rs, char db)
{
    volatile unsigned int i;

    // clear E
    GPIOPinWrite(GPIO_PORTM_BASE, GPIO_PIN_2, 0);
    for(i=0;i<LOOP_WRITE_TIMING;i++);

    // set RS, RW
    GPIOPinWrite(GPIO_PORTM_BASE, 0b11, 0b00+rs);
    for(i=0;i<LOOP_WRITE_TIMING;i++);

    // set E
    GPIOPinWrite(GPIO_PORTM_BASE, GPIO_PIN_2, GPIO_PIN_2);

    // Write DB
    GPIOPinWrite(GPIO_PORTK_BASE, 0xff, db);
    for(i=0;i<LOOP_WRITE_TIMING;i++);

    // clear E
    GPIOPinWrite(GPIO_PORTM_BASE, GPIO_PIN_2, 0);
    for(i=0;i<LOOP_WRITE_TIMING;i++);
}


void GLCD_clear()
{
    uint8_t cs, iy, ix;
    for(cs=0;cs<2;cs++) {
        if(cs==0) GLCD_CS(GLCD_CS1);
        if(cs==1) GLCD_CS(GLCD_CS2);
        GLCD_write(GLCD_CMD, GLCD_LINE+0);
        for(iy=0;iy<8;iy++)
        {
            GLCD_write(GLCD_CMD, GLCD_PAGE+iy);
            GLCD_write(GLCD_CMD, GLCD_ADDR+0);
            for(ix=0;ix<64;ix++) {
                GLCD_write(GLCD_DAT, 0b00000000);
            }
        }
        GLCD_write(GLCD_CMD, GLCD_LINE+0);
        for(iy=0;iy<8;iy++)
        {
            GLCD_write(GLCD_CMD, GLCD_PAGE+iy);
            GLCD_write(GLCD_CMD, GLCD_ADDR+0);
            for(ix=0;ix<64;ix++) {
                GLCD_write(GLCD_DAT, 0b00000000);
            }
        }
    }
}


void GLCD_char(int y, int x, char _c)
{
    int c;
    char i, n;

    if(_c<' ') _c = ' ';
    if(_c>'~') _c = ' ';
    c = (_c-32)*5+2;

    if(x>21) return;

    if(x<11) {
        GLCD_CS(GLCD_CS1);
        GLCD_write(GLCD_CMD, GLCD_PAGE+y);
        GLCD_write(GLCD_CMD, GLCD_ADDR+x*6);
        n = (x<10) ? 5 : 4;
        for(i=0;i<n;i++) {
            GLCD_write(GLCD_DAT, font5x7[c+i]);
        }
        if(n==5) GLCD_write(GLCD_DAT, 0);
    }

    if(x>9) {
        GLCD_CS(GLCD_CS2);
        GLCD_write(GLCD_CMD, GLCD_PAGE+y);
        n = (x<21) ? 5 : 2;
        if(x==10) {
            GLCD_write(GLCD_CMD, GLCD_ADDR);
            GLCD_write(GLCD_DAT, font5x7[c+4]);
        } else {
            GLCD_write(GLCD_CMD, GLCD_ADDR+(x-11)*6+2);
            for(i=0;i<n;i++) {
                GLCD_write(GLCD_DAT, font5x7[c+i]);
            }
        }
        if(n==5) GLCD_write(GLCD_DAT, 0);
    }
}



void GLCD_str(int y, int x, char *str)
{
    int i;
    int n = strlen(str);
    for(i=0;i<n;i++) {
        GLCD_char(y, x+i, str[i]);
    }
}




void GLCD_init()
{
    volatile unsigned int i;

    // I/O settings
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOG);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOH);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOK);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOM);
    GPIOPinTypeGPIOOutput(GPIO_PORTG_BASE, 0b00000010);
    GPIOPinTypeGPIOOutput(GPIO_PORTH_BASE, 0b00000011);
    GPIOPinTypeGPIOOutput(GPIO_PORTK_BASE, 0b11111111);
    GPIOPinTypeGPIOOutput(GPIO_PORTM_BASE, 0b00000111);

    // initialize GLCD
    GLCD_RST_H;

    // wait for more than 30 ms
    for(i=0;i<LOOP_WAIT_30MS;i++);

    // initialization steps
    GLCD_CS(GLCD_CS1);
    GLCD_write(GLCD_CMD, 0b11000000);
    GLCD_write(GLCD_CMD, 0b00111111);
    GLCD_CS(GLCD_CS2);
    GLCD_write(GLCD_CMD, 0b11000000);
    GLCD_write(GLCD_CMD, 0b00111111);
    GLCD_CS(0);

    // clear GLCD
    GLCD_clear();
}

